@@ -10,9 +10,9 @@
 
 __author__ = """Stephen Wasilewski"""
 __email__ = 'stephanwaz@gmail.com'
-__version__ = '1.0.4'
-__all__ = ['io', 'skycalc', 'translate', 'draw', 'lightfield',
-           'integrator', 'mapper', 'sampler', 'scene', 'renderer']
+__version__ = '1.1.0'
+__all__ = ['crenderer', 'evaluate', 'formatter', 'lightpoint', 'mapper',
+           'renderer', 'sampler', 'scene', 'sky', 'io', 'plot', 'translate']
 
 
 def set_raypath(basefile=__file__, subd="cal"):
