@@ -8,6 +8,7 @@
 # =======================================================================
 
 """functions for reading and writing"""
+from datetime import datetime, timezone
 import shlex
 import os
 import sys
@@ -18,8 +19,10 @@ from scipy.ndimage.filters import uniform_filter
 
 import numpy as np
 
+import raytraverse
 from raytraverse import translate
 from raytraverse.mapper import ViewMapper
+from raytraverse.crenderer import cRtrace
 
 
 class CaptureStdOut:
@@ -146,40 +149,6 @@ def unset_nproc():
         pass
 
 
-def call_sampler(outf, command, vecs, shape):
-    """make subprocess call to sampler given as command, expects rgb value
-    as return for each vec
-
-    Parameters
-    ----------
-    outf: str
-        path to write out to
-    command: str
-        command line with executable and options
-    vecs: np.array
-        vectors to pass as stdin to command
-    shape: tuple
-        shape of expected output
-
-    Returns
-    -------
-    lums: np.array
-        of length vectors.shape[0]
-
-    """
-    f = open(outf, 'a+b')
-    lum_file_pos = f.tell()
-    p = Popen(shlex.split(command), stdout=f, stdin=PIPE)
-    p.communicate(np2bytes(vecs))
-    lum = bytefile2rad(f, shape, subs='ijk,k->ij', offset=lum_file_pos)
-    return lum
-
-
-def bytefile2rad(f, shape, slc=..., subs='ijk,k->ij', offset=0):
-    memarray = np.memmap(f, dtype='<f', mode='r', shape=shape, offset=offset)
-    return np.einsum(subs, memarray[slc], [0.265, 0.670, 0.065])
-
-
 def np2bytes(ar, dtype='<f'):
     """format ar as bytestring
 
@@ -197,6 +166,23 @@ def np2bytes(ar, dtype='<f'):
     return ar.astype(dt).tobytes()
 
 
+def np2bytefile(ar, outf, dtype='<f'):
+    """save vectors to file
+
+    Parameters
+    ----------
+    ar: np.array
+        array to write
+    outf: str
+        file to write to
+    dtype: str
+        argument to pass to np.dtype()
+    """
+    f = open(outf, 'wb')
+    f.write(np2bytes(ar, dtype=dtype))
+    f.close()
+
+
 def bytes2np(buf, shape, dtype='<f'):
     """read ar from bytestring
 
@@ -236,21 +222,38 @@ def bytefile2np(f, shape, dtype='<f'):
     return bytes2np(f.read(), shape, dtype)
 
 
-def array2hdr(ar, imgf, header=None):
-    """write 2d np.array (x,y) to hdr image format
+def version_header():
+    """generate image header string"""
+    lastmod = os.path.getmtime(os.path.dirname(raytraverse.__file__))
+    lm = datetime.fromtimestamp(lastmod).strftime("%Y-%m-%d")
+    cap = datetime.now(timezone.utc).strftime("%Y-%m-%d %H:%M:%S %Z")
+    return ["CAPDATE= " + cap,
+            f"SOFTWARE= RAYTRAVERSE {raytraverse.__version__} lastmod {lm} //"
+            f" {cRtrace.version}"]
 
-    Parameters
-    ----------
-    ar: np.array
-    imgf: file path to right
-    header: list of header lines to append to image header
 
-    Returns
-    -------
+def _array2hdr(ar, imgf, header, pval):
+    """write 2d np.array to hdr image format
 
-    """
-    f = open(imgf, 'wb')
-    pval = f'pvalue -r -b -h -H -df -o -y {ar.shape[1]} +x {ar.shape[0]}'
+        Parameters
+        ----------
+        ar: np.array
+            image array
+        imgf: str
+            file path to right
+        header: list
+            list of header lines to append to image header
+        pval: str
+            pvalue command
+
+        Returns
+        -------
+        imgf
+        """
+    if imgf is None:
+        f = sys.stdout.buffer
+    else:
+        f = open(imgf, 'wb')
     if header is not None:
         hdr = "' '".join(header)
         getinfo = shlex.split(f"getinfo -a '{hdr}'")
@@ -259,12 +262,53 @@ def array2hdr(ar, imgf, header=None):
     else:
         p = Popen(pval.split(), stdin=PIPE, stdout=f)
         q = p
-    p.stdin.write(np2bytes(ar[-1::-1, -1::-1].T))
+    p.stdin.write(np2bytes(ar))
+    p.stdin.flush()
     q.communicate()
     f.close()
     return imgf
 
 
+def array2hdr(ar, imgf, header=None):
+    """write 2d np.array (x,y) to hdr image format
+
+    Parameters
+    ----------
+    ar: np.array
+            image array
+    imgf: str
+        file path to right
+    header: list
+        list of header lines to append to image header
+
+    Returns
+    -------
+    imgf
+    """
+    pval = f'pvalue -r -b -h -H -df -o -y {ar.shape[1]} +x {ar.shape[0]}'
+    return _array2hdr(ar[-1::-1, -1::-1].T, imgf, header, pval)
+
+
+def carray2hdr(ar, imgf, header=None):
+    """write color channel np.array (3, x, y) to hdr image format
+
+    Parameters
+    ----------
+    ar: np.array
+            image array
+    imgf: str
+        file path to right
+    header: list
+        list of header lines to append to image header
+
+    Returns
+    -------
+    imgf
+    """
+    pval = f'pvalue -r -h -H -df -o -y {ar.shape[-1]} +x {ar.shape[-2]}'
+    return _array2hdr(ar.T[-1::-1, -1::-1, :], imgf, header, pval)
+
+
 def uvarray2hdr(uvarray, imgf, header=None):
     res = uvarray.shape[0]
     vm = ViewMapper(viewangle=180)
@@ -277,35 +321,6 @@ def uvarray2hdr(uvarray, imgf, header=None):
     array2hdr(img.reshape(res, res), imgf, header)
 
 
-def carray2hdr(ar, imgf, header=None):
-    """write color channel np.array (3, x, y) to hdr image format
-
-    Parameters
-    ----------
-    ar: np.array
-    imgf: file path to right
-    header: list of header lines to append to image header
-
-    Returns
-    -------
-
-    """
-    f = open(imgf, 'wb')
-    pval = f'pvalue -r -h -H -df -o -y {ar.shape[-1]} +x {ar.shape[-2]}'
-    if header is not None:
-        hdr = "' '".join(header)
-        getinfo = shlex.split(f"getinfo -a '{hdr}'")
-        p = Popen(pval.split(), stdin=PIPE, stdout=PIPE)
-        q = Popen(getinfo, stdin=p.stdout, stdout=f)
-    else:
-        p = Popen(pval.split(), stdin=PIPE, stdout=f)
-        q = p
-    p.stdin.write(np2bytes(ar.T[-1::-1, -1::-1, :]))
-    q.communicate()
-    f.close()
-    return imgf
-
-
 def hdr2array(imgf):
     """read np.array from hdr image
 
