@@ -10,9 +10,110 @@
 """functions for translating between coordinate spaces and resolutions"""
 
 import numpy as np
-from scipy.interpolate import RectBivariateSpline
 from scipy.ndimage.filters import gaussian_filter, uniform_filter
 
+
+def norm(v):
+    """normalize 2D array of vectors along last dimension"""
+    return v/np.linalg.norm(v, axis=-1).reshape(-1, 1)
+
+
+def norm1(v):
+    """normalize flat vector"""
+    return v/np.sqrt(np.sum(np.square(v)))
+
+
+###############################################
+# Shirley=Chiu Disk to Square Transformations #
+###############################################
+
+def uv2xy(uv):
+    """translate from unit square (0,1),(0,1) to disk (x,y)
+    http://psgraphics.blogspot.com/2011/01/improved-code-for-concentric
+    -map.html.
+    """
+    uv = np.atleast_2d(uv)
+    np.seterr(all="ignore")
+    xy = np.empty((uv.shape[0], 2), np.float)
+    u = uv[:, 0]
+    v = uv[:, 1]
+    n = np.where(u > 1.0, -1, 1)
+    u = np.where(u > 1.0, u - 1.0, u)
+    a = 2. * u - 1
+    b = 2. * v - 1
+    cond = a*a > b*b
+    r = np.where(cond, a, np.where(b == 0, 0, b))
+    phi = np.where(cond, b/(2*a), np.where(b == 0, 0, 1 - a/(2*b)))*np.pi/2
+    xy[:, 0] = n*np.cos(phi)*r
+    xy[:, 1] = np.sin(phi)*r
+    return xy
+
+
+def uv2xyz(uv, axes=(0, 1, 2), xsign=-1):
+    """translate from 2 x unit square (0,2),(0,1) to unit sphere (x,y,z)
+    http://psgraphics.blogspot.com/2011/01/improved-code-for-concentric
+    -map.html.
+    """
+    np.seterr(all="ignore")
+    uv = np.atleast_2d(uv)
+    xyz = np.empty((uv.shape[0], 3), np.float)
+    u = uv[:, 0]
+    v = uv[:, 1]
+    # u > 1 values lay in the negative z hemisphere
+    n = np.where(u > 1.0, -1, 1)
+    # bring both hemispheres to (0,1)
+    u = np.where(u > 1.0, u - 1.0, u)
+    a = 2. * u - 1
+    b = 2. * v - 1
+    cond = a*a > b*b
+    r = np.where(cond, a, np.where(b == 0, 0, b))
+    phi = np.where(cond, b/(2*a), np.where(b == 0, 0, 1 - a/(2*b)))*np.pi/2
+    sphterm = r*np.sqrt(2 - r*r)
+    # flip back x in positive z space
+    xyz[:, axes[0]] = xsign*n*np.cos(phi)*sphterm
+    xyz[:, axes[1]] = np.sin(phi)*sphterm
+    # add sign to z
+    xyz[:, axes[2]] = n*(1 - r*r)
+    return xyz
+
+
+def xyz2uv(xyz, normalize=False, axes=(0, 1, 2), flipu=True):
+    """translate from vector x,y,z (normalized) to u,v (0,2),(0,1)
+    Shirley, Peter, and Kenneth Chiu. A Low Distortion Map Between Disk and
+    Square. Journal of Graphics Tools, vol. 2, no. 3, Jan. 1997, pp. 45-52.
+    Taylor and Francis+NEJM, doi:10.1080/10867651.1997.10487479.
+    """
+    xyz = np.atleast_2d(xyz)
+    if normalize:
+        xyz = norm(xyz)
+    uv = np.empty((xyz.shape[0], 2), np.float)
+    # store sign of z-axis to map both hemispheres as positive
+    n = np.where(xyz[:, axes[2]] < 0, -1, 1)
+    r2 = 1 - n*xyz[:, axes[2]]
+    x = xyz[:, axes[0]] / np.sqrt(2 - r2)
+    y = xyz[:, axes[1]] / np.sqrt(2 - r2)
+    r = np.sqrt(np.square(x) + np.square(y))
+    phi = np.arctan2(y, x)
+    pi4 = np.pi/4
+    phi = phi + np.where(phi < -pi4, 2*np.pi, 0)
+    a = np.where(phi < pi4, (r, phi*r/pi4),
+                 np.where(phi < 3*pi4, (-(phi - np.pi/2)*r/pi4, r),
+                          np.where(phi < 5*pi4, (-r, -(phi - np.pi)*r/pi4),
+                                   ((phi - 3*np.pi/2)*r/pi4, -r)))).T
+    # for the positive z-direction (n=1) map -1,1 to 1,0 (to correct flip)
+    # for the negative z-direction (n=-1) map -1,1 to 1,2
+    if flipu:
+        uv[:, 0] = (np.where(n < 0, 3, 1) - a[:, 0]*n) / 2.
+    else:
+        uv[:, 0] = (a[:, 0] + 2 - n) / 2.
+    uv[:, 1] = (a[:, 1] + 1) / 2.
+    return uv
+
+
+###########################################
+# Translate to/from shirley chiu sky bins #
+###########################################
+
 scbinscal = ("""
 { map U/V axis to bin divisions }
 axis(x) : mod(floor(side * x), side);
@@ -63,104 +164,32 @@ Dz = n * (1 - sq(r));
 """
 
 
-def norm(v):
-    """normalize 2D array of vectors along last dimension"""
-    return v/np.linalg.norm(v, axis=-1).reshape(-1, 1)
-
-
-def norm1(v):
-    """normalize flat vector"""
-    return v/np.sqrt(np.sum(np.square(v)))
-
-
-def tpnorm(thetaphi):
-    """normalize angular vector to 0-pi, 0-2pi"""
-    thetaphi[:, 0] = np.mod(thetaphi[:, 0] + np.pi, np.pi)
-    thetaphi[:, 1] = np.mod(thetaphi[:, 1] + 2*np.pi, 2*np.pi)
-    return thetaphi
-
-
-def uv2xy(uv):
-    """translate from unit square (0,1),(0,1) to disk (x,y)
-    http://psgraphics.blogspot.com/2011/01/improved-code-for-concentric
-    -map.html.
-    """
-    np.seterr(all="ignore")
-    xy = np.empty((uv.shape[0], 2), np.float)
-    u = uv[:, 0]
-    v = uv[:, 1]
-    n = np.where(u > 1.0, -1, 1)
-    u = np.where(u > 1.0, u - 1.0, u)
-    a = 2. * u - 1
-    b = 2. * v - 1
-    cond = a*a > b*b
-    r = np.where(cond, a, np.where(b == 0, 0, b))
-    phi = np.where(cond, b/(2*a), np.where(b == 0, 0, 1 - a/(2*b)))*np.pi/2
-    xy[:, 0] = n*np.cos(phi)*r
-    xy[:, 1] = np.sin(phi)*r
-    return xy
-
-
-def uv2xyz(uv, axes=(0, 1, 2), xsign=-1):
-    """translate from 2 x unit square (0,2),(0,1) to unit sphere (x,y,z)
-    http://psgraphics.blogspot.com/2011/01/improved-code-for-concentric
-    -map.html.
-    """
-    np.seterr(all="ignore")
-    xyz = np.empty((uv.shape[0], 3), np.float)
-    u = uv[:, 0]
-    v = uv[:, 1]
-    # u > 1 values lay in the negative z hemisphere
-    n = np.where(u > 1.0, -1, 1)
-    # bring both hemispheres to (0,1)
-    u = np.where(u > 1.0, u - 1.0, u)
-    a = 2. * u - 1
-    b = 2. * v - 1
-    cond = a*a > b*b
-    r = np.where(cond, a, np.where(b == 0, 0, b))
-    phi = np.where(cond, b/(2*a), np.where(b == 0, 0, 1 - a/(2*b)))*np.pi/2
-    sphterm = r*np.sqrt(2 - r*r)
-    # flip back x in positive z space
-    xyz[:, axes[0]] = xsign*n*np.cos(phi)*sphterm
-    xyz[:, axes[1]] = np.sin(phi)*sphterm
-    # add sign to z
-    xyz[:, axes[2]] = n*(1 - r*r)
-    return xyz
-
-
-def xyz2uv(xyz, normalize=False, axes=(0, 1, 2), flipu=True):
-    """translate from vector x,y,z (normalized) to u,v (0,2),(0,1)
-    Shirley, Peter, and Kenneth Chiu. A Low Distortion Map Between Disk and
-    Square. Journal of Graphics Tools, vol. 2, no. 3, Jan. 1997, pp. 45-52.
-    Taylor and Francis+NEJM, doi:10.1080/10867651.1997.10487479.
-    """
-    if normalize:
-        xyz = norm(xyz)
-    uv = np.empty((xyz.shape[0], 2), np.float)
-    # store sign of z-axis to map both hemispheres as positive
-    n = np.where(xyz[:, axes[2]] < 0, -1, 1)
-    r2 = 1 - n*xyz[:, axes[2]]
-    x = xyz[:, axes[0]] / np.sqrt(2 - r2)
-    y = xyz[:, axes[1]] / np.sqrt(2 - r2)
-    r = np.sqrt(np.square(x) + np.square(y))
-    phi = np.arctan2(y, x)
-    pi4 = np.pi/4
-    phi = phi + np.where(phi < -pi4, 2*np.pi, 0)
-    a = np.where(phi < pi4, (r, phi*r/pi4),
-                 np.where(phi < 3*pi4, (-(phi - np.pi/2)*r/pi4, r),
-                          np.where(phi < 5*pi4, (-r, -(phi - np.pi)*r/pi4),
-                                   ((phi - 3*np.pi/2)*r/pi4, -r)))).T
-    # for the positive z-direction (n=1) map -1,1 to 1,0 (to correct flip)
-    # for the negative z-direction (n=-1) map -1,1 to 1,2
-    if flipu:
-        uv[:, 0] = (np.where(n < 0, 3, 1) - a[:, 0]*n) / 2.
+def xyz2skybin(xyz, side, tol=0, normalize=False):
+    xyz = np.atleast_2d(xyz)
+    uv = xyz2uv(xyz, flipu=False, normalize=normalize)
+    if tol > 0:
+        tol = tol/side
+        uvi = np.linspace(-tol, tol, 3)
+        uvs = np.stack(np.meshgrid(uvi, uvi)).reshape(2, 9).T + uv
+        sbin = np.unique(uv2bin(uvs, side)).astype(int)
+        skybin = sbin[sbin < side**2]
     else:
-        uv[:, 0] = (a[:, 0] + 2 - n) / 2.
-    uv[:, 1] = (a[:, 1] + 1) / 2.
-    return uv
+        skybin = uv2bin(uv, side)
+    return skybin
 
 
+def skybin2xyz(bn, side):
+    uv = bin2uv(bn, side)
+    return uv2xyz(uv, xsign=1)
+
+
+##################################################
+# Translate to/from anglular fisheye projeection #
+##################################################
+
 def xyz2xy(xyz, axes=(0, 1, 2), flip=True):
+    """xyz coordinates to xy mapping of angular fisheye proejection"""
+    xyz = np.atleast_2d(xyz)
     r = np.arctan2(np.sqrt(np.sum(np.square(xyz[:, axes[0:2]]), -1)),
                    xyz[:, axes[2]])/(np.pi/2)
     phi = np.arctan2(xyz[:, axes[0]], xyz[:, axes[1]])
@@ -172,8 +201,14 @@ def xyz2xy(xyz, axes=(0, 1, 2), flip=True):
 
 
 def pxy2xyz(pxy, viewangle=180.0):
+    """pixel coordinates of angular fisheye to xyz"""
+    pxy = np.atleast_2d(pxy)
     pxy -= .5
-    pxy *= viewangle/180
+    # cl = theta2chord(np.pi/2)/(np.pi/2)
+    va = viewangle * np.pi / 360
+    clp = theta2chord(va)/va
+    # theta2chord(np.pi/2)/(np.pi/2)
+    pxy *= viewangle/180 * 0.9003163161571061 / clp
     d = np.sqrt(np.sum(np.square(pxy), -1))
     z = np.cos(np.pi*d)
     d = np.where(d <= 0, np.pi, np.sqrt(1 - z*z)/d)
@@ -182,8 +217,21 @@ def pxy2xyz(pxy, viewangle=180.0):
     return xyz
 
 
+##########################################
+# Translate to/from anglular coordinates #
+##########################################
+
+def tpnorm(thetaphi):
+    """normalize angular vector to 0-pi, 0-2pi"""
+    thetaphi = np.atleast_2d(thetaphi)
+    thetaphi[:, 0] = np.mod(thetaphi[:, 0] + np.pi, np.pi)
+    thetaphi[:, 1] = np.mod(thetaphi[:, 1] + 2*np.pi, 2*np.pi)
+    return thetaphi
+
+
 def tp2xyz(thetaphi, normalize=True):
     """calculate x,y,z vector from theta (0-pi) and phi (0-2pi) RHS Z-up"""
+    thetaphi = np.atleast_2d(thetaphi)
     if normalize:
         thetaphi = tpnorm(thetaphi)
     theta = thetaphi[:, 0]
@@ -196,6 +244,7 @@ def tp2xyz(thetaphi, normalize=True):
 
 def xyz2tp(xyz):
     """calculate theta (0-pi), phi from x,y,z RHS Z-up"""
+    xyz = np.atleast_2d(xyz)
     theta = np.arccos(xyz[:, 2])
     phi = np.where(np.isclose(theta, 0.0, atol=1e-10), np.pi,
                    np.where(np.isclose(theta, np.pi, atol=1e-10),
@@ -205,17 +254,73 @@ def xyz2tp(xyz):
 
 def tp2uv(thetaphi):
     """calculate UV from theta (0-pi), phi"""
+    thetaphi = np.atleast_2d(thetaphi)
     return xyz2uv(tp2xyz(thetaphi))
 
 
 def uv2tp(uv):
     """calculate theta (0-pi), phi from UV"""
+    uv = np.atleast_2d(uv)
     return xyz2tp(uv2xyz(uv))
 
 
-def uv2ij(uv, side):
+def aa2xyz(aa):
+    """calculate altitude (0-90), azimuth (-180,180) from xyz"""
+    aa = np.atleast_2d(aa)
+    tp = np.pi/2 - aa * np.pi/180
+    tp[:, 1] += np.pi
+    return tp2xyz(tp)
+
+
+def xyz2aa(xyz):
+    """calculate xyz from altitude (0-90), azimuth (-180,180)"""
+    xyz = np.atleast_2d(xyz)
+    tp = xyz2tp(xyz)
+    tp[:, 1] -= np.pi
+    return (np.pi/2 - tp)/(np.pi/180)
+
+
+def chord2theta(c):
+    """compute angle from chord on unit circle
+
+    Parameters
+    ----------
+    c: float
+        chord or euclidean distance between normalized direction vectors
+
+    Returns
+    -------
+    theta: float
+        angle captured by chord
+    """
+    return 2*np.arcsin(c/2)
+
+
+def theta2chord(theta):
+    """compute chord length on unit sphere from angle
+
+    Parameters
+    ----------
+    theta: float
+        angle
+
+    Returns
+    -------
+    c: float
+        chord or euclidean distance between normalized direction vectors
+    """
+    return 2*np.sin(theta/2)
+
+
+################################################
+# digitize and serialize UV square coordinates #
+################################################
+
+def uv2ij(uv, side, aspect=2):
+    uv = np.atleast_2d(uv)
     ij = np.mod(np.floor(side*uv), side)
-    ij[:, 0] += (uv[:, 0] >= 1) * side
+    if aspect == 2:
+        ij[:, 0] += (uv[:, 0] >= 1) * side
     return ij.astype(int)
 
 
@@ -224,18 +329,15 @@ def uv2bin(uv, side):
     return buv[:, 0]*side + buv[:, 1]
 
 
-def bin2uv(bn, side):
-    u = (bn - np.mod(bn, side))/(side*side)
-    v = np.mod(bn, side)/side
+def bin2uv(bn, side, offset=0.5):
+    u = ((bn - np.mod(bn, side))/side + offset)/side
+    v = (np.mod(bn, side) + offset)/side
     return np.stack((u, v)).T
 
 
-def bin_borders(sb, side):
-    si = np.stack(np.unravel_index(sb, (side, side)))
-    square = np.array([[0, 0], [0, 1], [1, 1], [1, 0]])/side
-    uv = np.repeat(si.T[:, None, :]/side, 4, axis=1) + square[None]
-    return uv
-
+#########################
+# image like resampling #
+#########################
 
 def resample(samps, ts=None, gauss=True, radius=None):
     """simple array resampling. requires whole number multiple scaling.
@@ -281,14 +383,9 @@ def resample(samps, ts=None, gauss=True, radius=None):
     return samps
 
 
-def interpolate2d(a, s):
-    oldcx = np.linspace(0, 1, a.shape[0])
-    newcx = np.linspace(0, 1, s[0])
-    oldcy = np.linspace(0, 1, a.shape[1])
-    newcy = np.linspace(0, 1, s[1])
-    f = RectBivariateSpline(oldcx, oldcy, a, kx=1, ky=1)
-    return f(newcx, newcy)
-
+####################
+# vector rotations #
+####################
 
 def rmtx_elem(theta, axis=2, degrees=True):
     if degrees:
@@ -348,47 +445,3 @@ def rmtx_yp(v):
                      (0, np.cos(p), -np.sin(p)),
                      (0, np.sin(p), np.cos(p))])
     return ymtx, pmtx
-
-
-def chord2theta(c):
-    """compute angle from chord on unit circle
-
-    Parameters
-    ----------
-    c: float
-        chord or euclidean distance between normalized direction vectors
-
-    Returns
-    -------
-    theta: float
-        angle captured by chord
-    """
-    return 2*np.arcsin(c/2)
-
-
-def theta2chord(theta):
-    """compute chord length on unit sphere from angle
-
-    Parameters
-    ----------
-    theta: float
-        angle
-
-    Returns
-    -------
-    c: float
-        chord or euclidean distance between normalized direction vectors
-    """
-    return 2*np.sin(theta/2)
-
-
-def aa2xyz(aa):
-    tp = np.pi/2 - aa * np.pi/180
-    tp[:, 1] += np.pi
-    return tp2xyz(tp)
-
-
-def xyz2aa(xyz):
-    tp = xyz2tp(xyz)
-    tp[:, 1] -= np.pi
-    return (np.pi/2 - tp)/(np.pi/180)
